      PROGRAM EBGF
C
C     PROGRAM EBGF (EXACT BALANCED GOODNESS-OF-FIT) COMPUTES THE EXACT
C     GOODNESS-OF-FIT PROBABILITY FOR FIVE STATISTICS:
C         (1) FISHER'S EXACT GOODNESS-OF-FIT TEST
C         (2) THE CHI-SQUARE GOODNESS-OF-FIT TEST
C         (3) THE ZELTERMAN GOODNESS-OF-FIT TEST
C         (4) THE LIKELIHOOD-RATIO GOODNESS-OF-FIT TEST
C         (5) THE CRESSIE-READ POWER DIVERGENT GOODNESS-OF-FIT TEST
C         (6) THE FREEMAN-TUKEY GOODNESS-OF-FIT TEST
C     AND THE ASYMPTOTIC GOODNESS-OF-FIT PROBABILITES FOR FOUR
C     STATISTICS:
C         (1) THE CHI-SQUARE GOODNESS-OF-FIT TEST
C         (2) THE ZELTERMAN GOODNESS-OF-FIT TEST
C         (3) THE LIKELIHOOD-RATIO GOODNESS-OF-FIT TEST
C         (4) THE WILLIAMS CORRECTED LIKELIHOOD-RATIO TEST
C         (5) THE SMITH ET AL. IMPROVED LIKELIHOOD-RATIO TEST
C
C     PROGRAM EBGF IS ONLY APPROPRIATE FOR EQUAL EXPECTED PROBABILITIES
C     IN EACH CATEGORY.
C
C     PROGRAM EBGF IS DIMENSIONED FOR NCAT = 50 CATEGORIES AND N = 120
C     OBSERVATIONS.  TO CHANGE THE DIMENSIONS, CHANGE THE VALUES OF 50
C     (5 TIMES) AND 121 (2 TIMES) IN THE DOUBLE PRECISION AND INTEGER
C     STATEMENTS BELOW.  ALSO, CHANGE THE VALUES OF LIMIT (120) AND
C     NCAT (50) IN THE DATA STATEMENT BELOW.
C
C     PROGRAM WRITTEN: 26 JULY 2003
C     PROGRAM REVISED: 20 AUGUST 2003
C
C     EXAMPLE: THE OUTPUT LISTED BELOW WAS GENERATED WITH THE FOLLOWING
C              14 OBSERVATIONS IN 8 CATEGORIES: 5 0 4 0 3 0 2 0.  THE
C              EXPECTED PROPORTIONS ARE EQUAL, I.E., 1/8 = 0.125.
C
C     ------------------------------------------------------------------
C     HEAD: EXAMPLE ANALYSIS WITH 8 CATEGORIES AND 14 OBSERVATIONS
C     OBSERVED VALUES: 5 0 4 0 3 0 2 0
C     ------------------------------------------------------------------
C     THE PROBABILITY OF THE OBSERVED TABLE:               .57355464D-06
C     FISHER'S EXACT PROBABILITY VALUE:                    .78065654D-02
C     THE OBSERVED CHI-SQUARE TEST STATISTIC:              .16857143D+02
C     THE EXACT CHI-SQUARE PROBABILITY VALUE:              .18538264D-01
C     THE ASYMPTOTIC CHI-SQUARE PROBABILITY VALUE:         .18340965D-01
C     THE OBSERVED ZELTERMAN TEST STATISTIC:               .16857143D+02
C     THE EXACT ZELTERMAN PROBABILITY VALUE:               .18538264D-01
C     THE ASYMPTOTIC ZELTERMAN PROBABILITY VALUE:          .
C     THE OBSERVED LIKELIHOOD-RATIO TEST STATISTIC:        .20879754D+02
C     THE EXACT LIKELIHOOD-RATIO PROBABILITY VALUE:        .50947990D-02
C     THE ASYMPTOTIC LIKELIHOOD-RATIO PROBABILITY VALUE:   .39522605D-02
C     THE WILLIAMS OBSERVED TEST STATISTIC:                .18859133D+02
C     THE WILLIAMS ASYMPTOTIC PROBABILITY VALUE:           .86402728D-02
C     THE SMITH ET AL. OBSERVED TEST STATISTIC:            .17975543D+02
C     THE ASYMPTOTIC SMITH ET AL. PROBABILITY VALUE:       .12080853D+01
C     THE OBSERVED FREEMAN-TUKEY TEST STATISTIC:           .19720526D+02
C     THE EXACT FREEMAN-TUKEY PROBABILITY VALUE:           .33374276D-02
C     THE ASYMPTOTIC FREEMAN-TUKEY PROBABILITY VALUE:      .62060613D-02
C     THE OBSERVED CRESSIE-READ(2/3) TEST STATISTIC:       .17084995D+02
C     THE EXACT CRESSIE-READ(2/3) PROBABILITY VALUE:       .12450064D-02
C     THE ASYMPTOTIC CRESSIE-READ(2/3) PROBABILITY VALUE:  .16856282D-02
C     ------------------------------------------------------------------
C     SUM OF ALL CONFIGURATION PROBABILITIES:                    1.00000
C     TOTAL NUMBER OF PARTITIONS:                                    135
C     NUMBER OF CONSISTENT PARTITIONS:                               116
C     CONFIGURATIONS CONSISTENT WITH MARGINAL TOTALS:             116280
C     CONFIGURATIONS YIELDING FISHER'S PROBABILITY:                39124
C     CONFIGURATIONS YIELDING THE CHI-SQUARE PROBABILITY:          49968
C     CONFIGURATIONS YIELDING THE ZELTERMAN PROBABILITY:           49968
C     CONFIGURATIONS YIELDING THE LIKELIHOOD-RATIO PROBABILITY:    33748
C     CONFIGURATIONS YIELDING THE FREEMAN-TUKEY PROBABILITY:       26300
C     CONFIGURATIONS YIELDING THE CRESSIE-READ(2/3) PROBABILITY:   45004
C     ------------------------------------------------------------------
      DOUBLE PRECISION APC,AZM,APCQ,APSM,APCR,APFT,APG,CHISQ,CHECK,ZMS,
     &                 CQ,CR,CRS,CS,ZM,CSM,FAC,FT,FTS,G2,P(50),PCR,PCS,
     &                 PZM,PFE,PFT,PL(50),PR,PT,PVN,PX2,PZL,SUMFE,SUMFT,
     &                 SUMCR,SUMCS,SUMZM,SUMLR,THISP,TOTAL,Z,ZL
      CHARACTER        FOUT*30, FMT2*30, HEAD*60
      INTEGER          ARRAY(2,121),BIN(121),FREQ(50),I,KOUNT,KZZ,LIMIT,
     &                 N,NCAT,O(50),Y(50)
      DATA             FMT2 /'(T5,A,  I3)'/, LIMIT /120/, NCAT /50/
C
C     CALL SUBROUTINE INPUT
C
      CALL INPUT (FOUT,HEAD,LIMIT,N,NCAT,O,P)
C
      DO 10 I = 1, NCAT
         PL(I) = DLOG(P(I))
   10 CONTINUE
C
C     CALCULATE THE EXACT PROBABILITY OF THE OBSERVED DATA (THISP),
C     THE OBSERVED TEST STATISTIC VALUES FOR CHI-SQUARE (CHISQ),
C     ZELTERMAN (ZMS), LIKELIHOOD-RATIO (G2), FREEMAN-TUKEY (FTS),
C     CRESSIE-READ (CRS), WILLIAMS CORRECTED LIKELIHOOD-RATIO (CQ),
C     AND SMITH ET AL. MODIFIED LIKELIHOOD-RATIO (CSM)
C
      Z = FAC(N)
      THISP = PR(NCAT,PL,O,Z)*(1.0D0-1.0D-12)
      CHISQ = CS(N,NCAT,O,P)*(1.0D0-1.0D-12)
      IF (CHISQ.LT.1.0D-11) CHISQ = -1.0D-11
      ZMS = ZM(N,NCAT,O,P)*(1.0D0-1.0D-12)
      IF (ZMS.LT.1.0D-11) ZMS = -1.0D-11
      G2 = ZL(N,NCAT,O,P)*(1.0D0-1.0D-12)
      IF (G2.LT.1.0D-11) G2 = -1.0D-11
      FTS = FT(N,NCAT,O,P)*(1.0D0-1.0D-12)
      IF (FTS.LT.1.0D-11) FTS = -1.0D-11
      CRS = CR(N,NCAT,O,P)*(1.0D0-1.0D-12)
      IF (CRS.LT.1.0D-11) CRS = -1.0D-11
      CQ = G2/(1.0D0+DBLE(NCAT+1)/DBLE(6*N))
      IF (CQ.LT.1.0D-11) CQ = -1.0D-11
      CSM = G2/(1.0D0+DBLE(NCAT+1)/DBLE(6*N)+DBLE(NCAT**2)/
     &      DBLE(6*N**2))
      IF (CSM.LT.1.0D-11) CSM = -1.0D-11
C
C     CALCULATE THE ASYMPTOTIC PROBABILITY VALUES
C
      PT = DEXP(THISP)
      IF (NCAT.EQ.2) THEN
         APC  = (PVN(CHISQ))**2
         AZM  = (PVN(ZMS))**2
         APG  = (PVN(G2))**2
         APCR = (PVN(CRS))**2
         APFT = (PVN(FTS))**2
         APCQ = (PVN(CQ))**2
         APSM = (PVN(CSM))**2
      ELSE
         APC  = PX2(NCAT-1,CHISQ)
         AZM  = PX2(NCAT-1,ZMS)
         APG  = PX2(NCAT-1,G2)
         APCR = PX2(NCAT-1,CRS)
         APFT = PX2(NCAT-1,FTS)
         APCQ = PX2(NCAT-1,CQ)
         APSM = PX2(NCAT-1,CSM)
      END IF
C
C     INITIALIZE THE COUNTERS FOR THE CHECKS ON THE CALCULATIONS
C
      SUMFE = 0.0D0
      SUMCR = 0.0D0
      SUMCS = 0.0D0
      SUMZM = 0.0D0
      SUMLR = 0.0D0
      SUMFT = 0.0D0
      CHECK = 0.0D0
      TOTAL = 0.0D0
      PFE   = 0.0D0
      PCS   = 0.0D0
      PZM   = 0.0D0
      PZL   = 0.0D0
      KOUNT = 0
      KZZ   = 0
C
C     CALL SUBROUTINE PART
C
      CALL PART (ARRAY,BIN,CHECK,CHISQ,ZMS,CRS,FREQ,FTS,G2,KOUNT,KZZ,N,
     &           NCAT,P,PCR,PCS,PZM,PFE,PFT,PL,PZL,SUMCR,SUMCS,SUMZM,
     &           SUMFE,SUMFT,SUMLR,THISP,TOTAL,Y,Z)
C
C     CALL SUBROUTINE OUTPUT
C
      CALL OUTPUT (APC,AZM,APCQ,APSM,APCR,APFT,APG,CHECK,CHISQ,ZMS,CQ,
     &             CSM,CRS,FTS,G2,HEAD,KOUNT,KZZ,NCAT,O,PCR,PCS,PZM,PFE,
     &             PFT,PT,PZL,SUMCR,SUMCS,SUMZM,SUMFE,SUMFT,SUMLR,TOTAL)
      END
C
      SUBROUTINE INPUT (FOUT,HEAD,LIMIT,N,NCAT,O,P)
C
C     SUBROUTINE INPUT READS IN VALUES FOR A HEADING, THE NAME OF AN
C     OUTPUT FILE, THE NUMBER OF CATEGORIES, THE OBSERVED FREQUENCY
C     FOR EACH CATEGORY.
C
      DOUBLE PRECISION P(NCAT)
      CHARACTER        FOUT*30,FMT1*30,HEAD*60
      INTEGER          I,LEN,LIMIT,N,NCAT,O(NCAT)
      DATA             FMT1 /'(T2,A,A,IX,A,$)'/
C
      WRITE  (*,10) 'E X A C T   G O O D N E S S   O F   F I T   ',
     &              'T E S T S'
   10 FORMAT (T14,A,A,/)
      WRITE  (*,20) 'NULL HYPOTHESIS: EQUALLY LIKELY CATEGORIES'
   20 FORMAT (T19,A,/)
      WRITE  (*,30) 'ENTER A HEADING FOR THE ANALYSIS: '
   30 FORMAT (T2,A,$)
      READ   (*,40)  HEAD
   40 FORMAT (A)
      WRITE  (*,30) 'ENTER A FILE NAME TO OUTPUT THE RESULTS: '
      READ   (*,40)  FOUT
      WRITE  (*,30) 'ENTER THE NUMBER OF CATEGORIES: '
      READ   (*,*)   NCAT
      LEN = 2
      IF (NCAT.LE.9) LEN = 1
      WRITE  (FMT1(10:10),50) LEN
   50 FORMAT (I1)
      WRITE  (*,FMT1) 'ENTER THE OBSERVATION VALUES FOR O(1), ',
     &                'O(2), ..., O(',NCAT,'): '
      READ   (*,*)    (O(I),I = 1,NCAT)
      N = 0
      DO 60 I = 1,NCAT
         N = N+O(I)
   60 CONTINUE
      IF (N.GT.LIMIT) THEN
         WRITE  (*,70) 'THE NUMBER OF OBSERVATIONS EXCEEDS THE ',
     &                 'DIMENSIONED ARRAYS OF THE PROGRAM.',
     &                 'INCREASE THE DIMENSIONS OF ARRAY, BIN, ',
     &                 'AND LIMIT IN THE MAIN PROGRAM.'
   70    FORMAT (/,T2,A,A,/,T2,A,A,/)
         STOP
      END IF
      P(1) = 1.0D0/DBLE(NCAT)
      DO 80 I = 2, NCAT
         P(I) = P(1)
   80 CONTINUE
      OPEN (UNIT = 9, FILE = FOUT, STATUS = 'NEW')
      RETURN
      END
C
C
      SUBROUTINE OUTPUT (APC,AZM,APCQ,APSM,APCR,APFT,APG,CHECK,CHISQ,
     &                   ZMS,CQ,CSM,CRS,FTS,G2,HEAD,KOUNT,KZZ,NCAT,O,
     &                   PCR,PCS,PZM,PFE,PFT,PT,PZL,SUMCR,SUMCS,SUMZM,
     &                   SUMFE,SUMFT,SUMLR,TOTAL)
C
C     SUBROUTINE OUTPUT PRINTS OUT THE RESULTS OF THE EXACT ANALYSES
C
      DOUBLE PRECISION APC,AZM,APCQ,APSM,APCR,APFT,APG,CHECK,CHISQ,ZMS,
     &                 CQ,CRIT,CSM,CRS,FTS,G2,PCR,PCS,PZM,PFE,PFT,PT,
     &                 PZL,SUMCR,SUMCS,SUMZM,SUMFE,SUMFT,SUMLR,TOTAL
      CHARACTER        FMT2*30,HEAD*60
      INTEGER          BIG,I,J,K,KOUNT,KZZ,LEN,NCAT,NUM,O(NCAT)
      DATA             FMT2 /'(T5,A,  I3)'/
C
      CRIT = 2.0D0**31-1.0D0
      WRITE  (9,10) 'FISHER, CHI-SQUARE, ZELTERMAN, ',
     &              'LIKELIHOOD-RATIO, FREEMAN-TUKEY,',
     &              'AND CRESSIE-READ(2/3) GOODNESS-OF-FIT ANALYSES'
   10 FORMAT (/,T14,A,A,/,T17,A,/)
      WRITE  (9,20) 'HEAD: ',HEAD
   20 FORMAT (T5,A,A)
C
      BIG = O(1)
      DO 30 I = 2,NCAT
         IF (O(I).GT.BIG) BIG = O(I)
   30 CONTINUE
      LEN = 2
      NUM = 26
      IF (BIG.GT.9) THEN
         LEN = 3
         NUM = 17
      END IF
      IF (BIG.GT.99) THEN
           LEN = 4
           NUM = 12
      END IF
      WRITE  (FMT2(10:10),40) LEN
   40 FORMAT (I1)
      IF (NCAT.LT.NUM) THEN
         WRITE  (FMT2(7:8),50) NCAT
   50    FORMAT (I2)
         WRITE  (9,FMT2) 'OBSERVED VALUES:', (O(I),I = 1,NCAT)
      ELSE
         J = NUM
         WRITE  (FMT2(7:8),50) J
         WRITE  (9,FMT2) 'OBSERVED VALUES:', (O(I),I = 1,J)
   60    K = NCAT-J
         IF (K.LE.0) GO TO 70
         IF (K.GT.NUM) THEN
            K = NUM
            J = J+K
         ELSE
            J = J+K
         END IF
         WRITE  (FMT2(7:8),50) K
         WRITE  (9,FMT2) '                ', (O(I),I = J-K+1,J)
         GO TO 60
      END IF
   70 WRITE  (9,80)
   80 FORMAT (T5,71('-'))
      WRITE  (9,90)  'THE PROBABILITY OF THE OBSERVED TABLE: ',PT
   90 FORMAT (T5,A,T61,D15.8)
      WRITE  (9,90)  'FISHER''S EXACT PROBABILITY VALUE: ',PFE
      IF (CHISQ.LT.0.0D0) CHISQ = 0.0D0
      WRITE  (9,90)  'THE OBSERVED CHI-SQUARE TEST STATISTIC: ',
     &                CHISQ
      WRITE  (9,90)  'THE EXACT CHI-SQUARE PROBABILITY VALUE: ',
     &                PCS
      WRITE  (9,90)  'THE ASYMPTOTIC CHI-SQUARE PROBABILITY VALUE:',
     &                APC
      IF (ZMS.LT.0.0D0) ZMS = 0.0D0
      WRITE  (9,90)  'THE OBSERVED ZELTERMAN TEST STATISTIC: ',
     &                ZMS
      WRITE  (9,90)  'THE EXACT ZELTERMAN PROBABILITY VALUE: ',
     &                PZM
      WRITE  (9,90)  'THE ASYMPTOTIC ZELTERMAN PROBABILITY VALUE:',
     &                AZM
      IF (G2.LT.0.0D0) G2 = 0.0D0
      WRITE  (9,90)  'THE OBSERVED LIKELIHOOD-RATIO TEST STATISTIC:',
     &                G2
      WRITE  (9,90)  'THE EXACT LIKELIHOOD-RATIO PROBABILITY VALUE:',
     &                PZL
      WRITE  (9,100) 'THE ASYMPTOTIC LIKELIHOOD-RATIO PROBABILITY ',
     &               'VALUE: ',APG
  100 FORMAT (T5,A,A,T61,D15.8)
      WRITE  (9,90)  'THE WILLIAMS OBSERVED TEST STATISTIC:',
     &                CQ
      WRITE  (9,90)  'THE WILLIAMS ASYMPTOTIC PROBABILITY VALUE:',
     &                APCQ
      WRITE  (9,90)  'THE SMITH ET AL. OBSERVED TEST STATISTIC:',
     &                CSM
      WRITE  (9,100) 'THE SMITH ET AL. ASYMPTOTIC PROBABILITY ',
     &               'VALUE:',APSM
      WRITE  (9,90)  'THE OBSERVED FREEMAN-TUKEY TEST STATISTIC:',
     &                FTS
      WRITE  (9,90)  'THE EXACT FREEMAN-TUKEY PROBABILITY VALUE:',
     &                PFT
      WRITE  (9,90)  'THE ASYMPTOTIC FREEMAN-TUKEY PROBABILITY VALUE:',
     &                APFT
      WRITE  (9,90)  'THE OBSERVED CRESSIE-READ(2/3) TEST STATISTIC:',
     &                CRS
      WRITE  (9,90)  'THE EXACT CRESSIE-READ(2/3) PROBABILITY VALUE:',
     &                PCR
      WRITE  (9,100) 'THE ASYMPTOTIC CRESSIE-READ(2/3) PROBABILITY ',
     &               'VALUE:',APCR
      WRITE  (9,80)
C
C     THE STATEMENTS BELOW CAN BE USED TO CHECK ON THE RESULTS
C     REMOVE THE "C" IN COLUMN 1 DOWN TO STATEMENT "END".
C
      WRITE  (9,120) 'SUM OF ALL CONFIGURATION PROBABILITIES:',
     &                SNGL(CHECK)
  120 FORMAT (T5,A,T68,F8.5)
      WRITE  (9,130) 'TOTAL NUMBER OF PARTITIONS:', KOUNT
  130 FORMAT (T5,A,T64,I12)
C
      WRITE  (9,130) 'NUMBER OF CONSISTENT PARTITIONS:', KZZ
C
      IF (TOTAL.LT.CRIT) THEN
         WRITE  (9,140) 'CONFIGURATIONS CONSISTENT WITH MARGINAL ',
     &                  'TOTALS:', IDNINT(TOTAL)
  140    FORMAT (T5,A,A,T64,I12)
      ELSE
         WRITE  (9,150) 'CONFIGURATIONS CONSISTENT WITH MARGINAL ',
     &                  'TOTALS:', TOTAL
  150    FORMAT (T5,A,A,T64,D12.6)
      END IF
C
      IF (SUMFE.LT.CRIT) THEN
         WRITE  (9,140) 'CONFIGURATIONS YIELDING FISHER''S ',
     &                  'PROBABILITY:', IDNINT(SUMFE)
      ELSE
         WRITE  (9,150) 'CONFIGURATIONS YIELDING FISHER''S ',
     &                  'PROBABILITY:', SUMFE
      END IF
C
      IF (SUMCS.LT.CRIT) THEN
         WRITE  (9,140) 'CONFIGURATIONS YIELDING THE CHI-SQUARE ',
     &                  'PROBABILITY:', IDNINT(SUMCS)
      ELSE
         WRITE  (9,150) 'CONFIGURATIONS YIELDING THE CHI-SQUARE ',
     &                  'PROBABILITY:', SUMCS
      END IF
C
      IF (SUMZM.LT.CRIT) THEN
         WRITE  (9,140) 'CONFIGURATIONS YIELDING THE ZELTERMAN ',
     &                  'PROBABILITY:', IDNINT(SUMZM)
      ELSE
         WRITE  (9,150) 'CONFIGURATIONS YIELDING THE ZELTERMAN ',
     &                  'PROBABILITY:', SUMZM
      END IF
C
      IF (SUMLR.LT.CRIT) THEN
         WRITE  (9,140) 'CONFIGURATIONS YIELDING THE LIKELIHOOD-RATIO ',
     &                  'PROBABILITY:', IDNINT(SUMLR)
      ELSE
         WRITE  (9,150) 'CONFIGURATIONS YIELDING THE LIKELIHOOD-RATIO ',
     &                  'PROBABILITY:', SUMLR
      END If
C
      IF (SUMFT.LT.CRIT) THEN
         WRITE  (9,140) 'CONFIGURATIONS YIELDING THE FREEMAN-TUKEY ',
     &                  'PROBABILITY:', IDNINT(SUMFT)
      ELSE
         WRITE  (9,150) 'CONFIGURATIONS YIELDING THE FREEMAN-TUKEY ',
     &                  'PROBABILITY:', SUMFT
      END IF
C
      IF (SUMCR.LT.CRIT) THEN
         WRITE  (9,140) 'CONFIGURATIONS YIELDING THE CRESSIE-READ ',
     &                  'PROBABILITY:', IDNINT(SUMCR)
      ELSE
         WRITE  (9,150) 'CONFIGURATIONS YIELDING THE CRESSIE-READ ',
     &                  'PROBABILITY:', SUMCR
      END IF
C
      WRITE  (9,80)
      RETURN
      END
C
C
      SUBROUTINE PART (ARRAY,BIN,CHECK,CHISQ,ZMS,CRS,FREQ,FTS,G2,KOUNT,
     &                 KZZ,N,NCAT,P,PCR,PCS,PZM,PFE,PFT,PL,PZL,SUMCR,
     &                 SUMCS,SUMZM,SUMFE,SUMFT,SUMLR,THISP,TOTAL,Y,Z)
C
C     SUBROUTINE PART GENERATES ALL POSSIBLE PARTITIONS OF THE SET
C     {1, 2, 3, ..., N}.
C
      DOUBLE PRECISION CHECK,CHISQ,ZMS,CRS,FTS,G2,P(NCAT),PCR,PCS,PZM,
     &                 PFE,PFT,PL(NCAT),PZL,SUMCR,SUMCS,SUMZM,SUMFE,
     &                 SUMFT,SUMLR,THISP,TOTAL,Z
      INTEGER          ARRAY(2,N+1),BIN(N+1),FREQ(NCAT),J,K,KOUNT,
     &                 KZZ,LIMIT,N,NCAT,TEST,TOT,Y(NCAT)
C
      DO 10 K = 1,N
         ARRAY(1,K) = 0
         ARRAY(2,K) = 0
   10 CONTINUE
      J = 1
      ARRAY(1,J) = N
      ARRAY(2,J) = N
   20 KOUNT = KOUNT+1
C
C     CALL SUBROUTINE JOB
C
      CALL JOB (ARRAY,BIN,KOUNT,N,NCAT,LIMIT)
      IF (LIMIT-1.LE.NCAT) THEN
C
C        CALL SUBROUTINE CALC
C
         CALL CALC (BIN,CHECK,CHISQ,ZMS,CRS,FREQ,FTS,G2,KZZ,N,NCAT,P,
     &              PCR,PCS,PZM,PFE,PFT,PL,PZL,SUMCR,SUMCS,SUMZM,SUMFE,
     &              SUMFT,SUMLR,THISP,TOTAL,Y,Z)
      END IF
C
      IF (ARRAY(2,N).EQ.1) GO TO 70
      DO 30 K = 1,N
         ARRAY(1,K) = ARRAY(2,K)
         ARRAY(2,K) = 0
   30 CONTINUE
      TEST = ARRAY(1,J)-2
      IF (TEST.GE.0) THEN
         ARRAY(2,J) = TEST
         J = J+1
         ARRAY(2,J) = ARRAY(1,J)+1
         DO 40 K = J+1,N
            ARRAY(2,K) = ARRAY(1,K)
   40    CONTINUE
         J = 1
         GO TO 20
      ELSE
         TOT = ARRAY(1,1)
         ARRAY(2,1) = 0
         DO 60 K = 2,N
            IF (TOT.LT.K) THEN
               TOT = TOT+K*ARRAY(1,K)
               ARRAY(2,K) = 0
            ELSE
               ARRAY(2,K) = ARRAY(1,K)+1
               ARRAY(2,1) = TOT-K
               DO 50 J = K+1,N
                  ARRAY(2,J) = ARRAY(1,J)
   50          CONTINUE
               GO TO 20
            END IF
   60    CONTINUE
      END IF
   70 RETURN
      END
C
C
      SUBROUTINE JOB (ARRAY,BIN,KOUNT,N,NCAT,LIMIT)
C
C     SUBROUTINE JOB CONVERTS THE TABLE GENERATED IN SUBROUTINE PART
C     INTO THE DESIRED PARTITION ARRAYS.
C
      CHARACTER FMT3*30
      INTEGER   ARRAY(2,N+1),BIN(N+1),I,J,K,KOUNT,LIMIT,M,N,NCAT
      DATA      FMT3 /'(I9,I5,  I3) '/
C
      DO 10 I = 1,N
         BIN(I) = 0
   10 CONTINUE
      K = 1
      LIMIT = 1
   20 M = ARRAY(2,K)
      IF (M.EQ.0) GO TO 40
      DO 30 J = LIMIT, LIMIT+M-1
         BIN(J) = K
   30 CONTINUE
      IF (ARRAY(2,N).EQ.1) LIMIT = LIMIT+1
   40 K = K+1
      IF (K.LE.N) THEN
         LIMIT = LIMIT+M
         GO TO 20
      END IF
      IF (LIMIT-1.LE.NCAT) THEN
C
C     THE FOLLOWING STATEMENTS PROVIDE A MEANS TO LIST ALL PARTITIONS
C     REMOVE THE "C" IN COLUMN 1 OF THE NEXT 3 ROWS
C
C        WRITE (FMT3(8:9),50) LIMIT-1
C  50    FORMAT (I2)
C        WRITE  (*,FMT3) KOUNT, (BIN(J),J = 1, LIMIT-1)
      END IF
      RETURN
      END
C
C
      SUBROUTINE CALC (BIN,CHECK,CHISQ,ZMS,CRS,FREQ,FTS,G2,KZZ,N,NCAT,
     &                 P,PCR,PCS,PZM,PFE,PFT,PL,PZL,SUMCR,SUMCS,SUMZM,
     &                 SUMFE,SUMFT,SUMLR,THISP,TOTAL,Y,Z)
C
C     SUBROUTINE CALC CALCULATES THE EXACT PROBABILITY VALUES FOR
C     FISHER'S EXACT (PFE), CHI-SQUARE (PCS), ZELTERMAN (PZM),
C     LIKELIHOOD-RATIO (PZL), FREEMAN-TUKEY (PFT), AND CRESSIE-READ (PCR)
C     GOODNESS-OF-FIT TESTS
C
      DOUBLE PRECISION CC,ZT,CF,CG,CHECK,CHISQ,ZMS,CP,CR,CRS,CS,CZ,EXCP,
     &                 FT,FTS,G2,MULT,P(NCAT),PCR,PCS,PFE,PZM,PFT,
     &                 PL(NCAT),PR,PZL,SUMCR,SUMCS,SUMZM,SUMFE,SUMFT,
     &                 SUMLR,THISP,TOTAL,WEIGHT,Z,ZL
      INTEGER          BIN(N+1),FREQ(NCAT),KZZ,N,NCAT,Y(NCAT)
C
C     FISHER'S EXACT PROBABILITY
C
      MULT = WEIGHT(BIN,FREQ,N,NCAT,Y)
      KZZ = KZZ+1
      TOTAL = TOTAL+MULT
      CP = PR(NCAT,PL,BIN,Z)
      EXCP = DEXP(CP)
      CHECK = CHECK+MULT*EXCP
      IF (CP.LT.THISP) THEN
         PFE = PFE+MULT*EXCP
         SUMFE = SUMFE+MULT
      END IF
C
C     CHI-SQUARE EXACT PROBABILITY
C
      CC = CS(N,NCAT,BIN,P)
      IF (CC.GT.CHISQ) THEN
         PCS = PCS+MULT*EXCP
         SUMCS = SUMCS+MULT
      END IF
C
C     ZELTERMAN EXACT PROBABILITY
C
      ZT = ZM(N,NCAT,BIN,P)
      IF (ZT.GT.ZMS) THEN
         PZM = PZM+MULT*EXCP
         SUMZM = SUMZM+MULT
      END IF
C
C     LIKELIHOOD-RATIO EXACT PROBABILITY
C
      CG = ZL(N,NCAT,BIN,P)
      IF (CG.GT.G2) THEN
         PZL = PZL+MULT*EXCP
         SUMLR = SUMLR+MULT
      END IF
C
C     FREEMAN-TUKEY EXACT PROBABILITY
C
      CF = FT(N,NCAT,BIN,P)
      IF (CF.GT.FTS) THEN
         PFT = PFT+MULT*EXCP
         SUMFT = SUMFT+MULT
      END IF
C
C     CRESSIE-READ EXACT PROBABILITY
C
      CZ = CR(N,NCAT,BIN,P)
      IF (CZ.GT.CRS) THEN
         PCR = PCR+MULT*EXCP
         SUMCR = SUMCR+MULT
      END IF
C
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION FAC (I)
C
C     FUNCTION FAC CALCULATES LOG(I!)
C
      DOUBLE PRECISION  W, PREV(2500)
      INTEGER           I, J, MAX
      DATA              MAX /1/
C
C     INITIALIZE LOG(0!) = LOG(1) = 0
C
      FAC = 0.0D0
      PREV(1) = 0.0D0
      IF (I.LE.1) RETURN
C
C     CHECK TO SEE IF CALCULATION HAS ALREADY BEEN DONE
C
      IF (I.LE.MAX) THEN
         FAC = PREV(I)
         RETURN
      END IF
      FAC = PREV(MAX)
      DO 10 J = MAX+1,I
         W = DBLE(J)
         FAC = FAC+DLOG(W)
         PREV(J) = FAC
   10 CONTINUE
      MAX = I
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION PR (NCAT,PL,O,Z)
C
C     FUNCTION PR CALCULATES THE SUM OF LOG PROBABILITIES FOR
C     ALL CATEGORIES
C
      DOUBLE PRECISION FAC,PL(NCAT),Z
      INTEGER          I,NCAT,O(NCAT)
C
      PR = Z
      DO 10 I = 1,NCAT
         PR = PR-FAC(O(I))+O(I)*PL(I)
   10 CONTINUE
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION CS (N,NCAT,O,P)
C
C     FUNCTION CS CALCULATES THE CHI-SQUARED TEST STATISTIC
C
      DOUBLE PRECISION  P(NCAT)
      INTEGER           I,O(NCAT),N,NCAT
      CS = 0.0D0
      DO 10 I = 1, NCAT
         CS = CS+DBLE(O(I))**2/P(I)
   10 CONTINUE
      CS = CS/DBLE(N)-DBLE(N)
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION ZM (N,NCAT,O,P)
C
C     FUNCTION ZM CALCULATES THE ZELTERMAN TEST STATISTIC
C
      DOUBLE PRECISION  P(NCAT)
      INTEGER           I,O(NCAT),N,NCAT
      ZM = 0.0D0
      DO 10 I = 1, NCAT
         ZM = ZM+DBLE(O(I)*(O(I)-1))/P(I)
   10 CONTINUE
      ZM = ZM/DBLE(N)+DBLE(NCAT)-DBLE(N)
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION ZL (N,NCAT,O,P)
C
C     FUNCTION ZL CALCULATES THE LIKELIHOOD-RATIO TEST STATISTIC
C
      DOUBLE PRECISION P(NCAT),RZ,X
      INTEGER          I,N,NCAT,O(NCAT)
C
      ZL = 0.0D0
      DO 10 I = 1, NCAT
         X = DBLE(O(I))
         IF (X.GT.1.5D0) THEN
            RZ = X*DLOG(X)
         ELSE
            RZ = 0.0D0
         END IF
         ZL = ZL+RZ-DLOG(P(I)*DBLE(N))*X
   10 CONTINUE
      ZL = 2.0D0*ZL
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION FT (N,NCAT,O,P)
C
C     FUNCTION FT CALCULATES THE FREEMAN-TUKEY TEST STATISTIC
C
      DOUBLE PRECISION P(NCAT),X
      INTEGER          I,N,NCAT,O(NCAT)
C
      FT = 0.0D0
      DO 10 I = 1,NCAT
         X = DBLE(O(I))
         FT = FT+(DSQRT(X)+DSQRT(X+1.0D0)-DSQRT(P(I)*DBLE(N)*4.0D0
     &        +1.0D0))**2
   10 CONTINUE
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION CR (N,NCAT,O,P)
C
C     FUNCTION CR CALCULATES THE CRESSIE-READ (2/3) TEST STATISTIC
C
      DOUBLE PRECISION A,P(NCAT),X
      INTEGER          I,N,NCAT,O(NCAT)
C
      A = 2.0D0/3.0D0
      CR = 0.0D0
      DO 10 I = 1,NCAT
         X = DBLE(O(I))
         CR = CR+X*((X/(P(I)*DBLE(N)))**A-1.0D0)
   10 CONTINUE
      CR = CR*2.0D0/(A*(A+1.0D0))
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION WEIGHT (BIN,FREQ,N,NCAT,Y)
C
C     FUNCTION WEIGHT OBTAINS THE MULTINOMIAL WEIGHTS FOR THE
C     PROBABILITY VALUES
C
      DOUBLE PRECISION FAC,SUM
      INTEGER          BIN(N+1),COUNT,FREQ(NCAT),J,K,N,NCAT,Y(NCAT)
C
C     CREATE A FREQUENCY DISTRIBUTION IN Y(I) AND FREQ(I)
C
      DO 10 J = 1,NCAT
            FREQ(J) = 0
   10 CONTINUE
      COUNT = 1
      K = 1
      Y(K) = BIN(1)
      FREQ(K) = 1
      DO 20 J = 2,NCAT
         Y(J) = BIN(J)
         IF (Y(J).EQ.Y(K)) THEN
            FREQ(K) = FREQ(K)+1
         ELSE
            K = K+1
            Y(K) = Y(J)
            FREQ(K) = FREQ(K)+1
            COUNT = COUNT+1
         END IF
   20 CONTINUE
C
C     CALCULATE THE MULTINOMIAL COEFFICIENTS
C
      SUM = 0.0D0
      DO 30 J = 1,COUNT
         SUM = SUM+FAC(FREQ(J))
   30 CONTINUE
      WEIGHT = DEXP(FAC(NCAT)-SUM)
      RETURN
      END
C
C
C
      DOUBLE PRECISION FUNCTION PX2 (DF,T)
C
C     SUBROUTINE PX2 CALCULATES THE ASYMPTOTIC PROBABILITY OF A VALUE
C     OF T BEING GREATER THAN OR EQUAL TO THE OBSERVED VALUE OF T
C     USING THE CHI-SQUARED DISTRIBUTION
C
      DOUBLE PRECISION A,B,E1,E2,E3,E4,E5,H,PI,T,V,W
      INTEGER          DF,I,J,K,L
      DATA             E1 /  0.319381530D0 /, E2 / -0.356563782D0 /,
     &                 E3 /  1.781477937D0 /, E4 / -1.821255978D0 /,
     &                 E5 /  1.330274429D0 /,  H /  0.231641900D0 /
C
      IF (T.LT.0.0D0) THEN
         PX2 = 1.0D0
         RETURN
      END IF
      V = DBLE(DF)
      IF (V/2.0D0-DF/2.GT.0.1D0) GO TO 30
      PX2 = DEXP(-T/2.0D0)
      IF (DF.LT.3) RETURN
      K = (DF-2)/2
      B = 1.0D0
      DO 20 J = 1,K
         L = J
         A = 1.0D0
         DO 10 I = 1,L
            A = A*T/DBLE(2*I)
   10    CONTINUE
         B = B+A
   20 CONTINUE
      PX2 = PX2*B
      RETURN
   30 PI = 4.0D0*DATAN(1.0D0)
      W = 1.0D0/(H*DSQRT(T)+1.0D0)
      PX2 = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*DEXP(-T/2.0D0)
     &      /DSQRT(PI/2.0D0)
      IF (DF.LT.2) RETURN
      K = (DF-1)/2
      B = 0.0D0
      DO 50 J = 1,K
         L = J
         A = 1.0D0
         DO 40 I = 1,L
            A = A*T/DBLE(2*I-1)
   40    CONTINUE
         B = B+A
   50 CONTINUE
      PX2 = PX2+B*DEXP(-T/2.0D0)/DSQRT(PI*T/2.0D0)
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION PVN (Z)
C
C     FUNCTION PVN CALCULATES THE PROBABILITY OF A VALUE OF Z BEING
C     GREATER THAN OR EQUAL TO THE OBSERVED VALUE OF Z USING THE
C     NORMAL DISTRIBUTION
C
      DOUBLE PRECISION A,B,C,E1,E2,E3,E4,E5,EE,G,H,PI,U,W,Z
      DATA             E1 / 0.319381530D0/, E2 /-0.356563782D0/,
     &                 E3 / 1.781477937D0/, E4 /-1.821255978D0/,
     &                 E5 / 1.330274429D0/,  H / 0.231641900D0/
C
      PI = 4.0D0*DATAN(1.0D0)
      PVN = 0.0D0
      G = -Z*Z/2.0D0
      IF (G.LT.-709.1D0) GO TO 20
      A = 4.2D0
      U = DABS(Z)
      IF (U.GT.A) GO TO 10
      W = 1.0D0/(H*DABS(Z)+1.0D0)
      PVN = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W+EE(G)/DSQRT(PI*2.0D0)
      GO TO 20
   10 B = 2.2D0
      C = 4.9D0
      PVN = EE(G)*(1.0D0/U-1.0D0/U**3+B/U**C)/DSQRT(PI/2.0D0)
   20 IF (Z.LT.0.0D0) PVN = 1.0D0-PVN
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION EE(G)
C
      DOUBLE PRECISION G
C
      EE = 0.0D0
      IF (G.GE.-709.1D0) EE = DEXP(G)
      RETURN
      END
